from pathlib import Path
from ruamel.yaml import YAML


CONFIG_FOLDER = Path.home() / '.config' / 'jrnlsync'
CONFIG_PATH = CONFIG_FOLDER / 'jrnlsync.yaml'

JRNL_CONFIG_FOLDER = Path.home() / '.config' / 'jrnl'
JRNL_CONFIG_PATH = JRNL_CONFIG_FOLDER / 'jrnl.yaml'

yaml = YAML()


def read_config(path=None):
    
    # use same configuration folder approach as jrnlsync
    config_path = path or CONFIG_PATH

    if config_path.exists():
        with open(config_path, 'r') as f:
            return yaml.load(f)
    else:
        # TODO raise config not found error (need to create setup)
        return {}

def read_jrnl_config(path=None):
    config_path = path or JRNL_CONFIG_PATH

    if config_path.exists():
        with open(config_path, 'r') as f:
            return yaml.load(f)
    else:
        # TODO raise config not found error 
        return {}
    


def create_default_config():
    # create config folder if it doesn't exist
    CONFIG_FOLDER.mkdir(parents=True, exist_ok=True)

    default_jrnl_config_path = Path.home() / '.config' / 'jrnl'

    default_config = {
        "jrnl_folder": str(default_jrnl_config_path.absolute()),
    }
    yaml.dump(default_config, CONFIG_PATH)



from .config import read_config, create_default_config, read_jrnl_config

import sys
import subprocess


def git(commands, cwd=None):
    config = read_config()
    jrnl_folder = cwd or config.get("jrnl_folder")
    return subprocess.run(["git"] + commands, cwd=jrnl_folder)


def sync_journals(commit_message="jnrlsync automated commit"):
    git(["pull"])
    git(["commit", "-am", commit_message])
    git(["push"])
    return


def main():
    args = sys.argv[1:]
    
    if not args:
        sync_journals()

    elif args:

        if args[0] == "--start":
            create_default_config()
            return

        if args[0] == "git":
            args.pop(0)
            p = git(args)
            return p
        
        if args[0] in ("-m", "--message"):
            try:
                commit_message = args[1]
            except IndexError:
                commit_message = input("Please specify commit message: ")
            sync_journals(commit_message=commit_message)
            return
        
        if args[0] == "open":
            try:
                jrnl = args[1]
            except IndexError:
                jrnl = 'default'
            jrnl_config = read_jrnl_config()
            print(f"Open jrnl file for {jrnl}")
            journals = jrnl_config.get("journals")
            journal_path = journals.get(jrnl)
            editor = jrnl_config.get("editor")
            p = subprocess.run([editor, journal_path])
            return p 
        
        else:
            p = git(args)
            return p
        return



import click
import subprocess
from .config import read_config, read_jrnl_config, create_default_config


@click.group(invoke_without_command=True)
@click.pass_context
@click.option("-m", "--message", default="jrnlsync auto commit", type=str)
@click.argument('args', nargs=-1, type=str)
def cli(ctx, message, args):

    config = read_config()
    jrnl_folder = config.get("jrnl_folder")
    
    if ctx.invoked_subcommand is None:
        click.echo('I was invoked without subcommand')
        # TODO: continue with git logic:
        if not args:
            click.echo("no args")
            # \-> no commands: sync using git
            p = subprocess.run(["git", "pull"], cwd=jrnl_folder)
            p = subprocess.run(["git", "commit", "-am", message], cwd=jrnl_folder)
            p = subprocess.run(["git", "push"], cwd=jrnl_folder)

        elif args:
            # \-> with commands: assume git command
            click.echo(f"with args {args}")
            p = subprocess.run(["git"] + list(args), cwd=jrnl_folder)
            return p
        else:
            # This should not happen
            raise Exception("Something strange is happening")
        
    else:
        click.echo(f"I am about to invoke {ctx.invoked_subcommand}")
        # TODO: create subcommand logic
    return

@cli.command()
@click.argument('args', nargs=-1, type=str)
def git(args):
    config = read_config()
    jrnl_folder = config.get("jrnl_folder")

    p = subprocess.run(["git"] + list(args), cwd=jrnl_folder)
    return

@cli.command()
@click.argument("journal", default="default", type=str)
def open(journal):
    # TODO: continue with open command
    jrnl_config = read_jrnl_config()
    click.echo(f"Open jrnl file for {journal}")
    journals = jrnl_config.get("journals")
    journal_path = journals.get(journal)
    click.echo(f"journal path: {journal_path}")
    editor_path = jrnl_config.get("editor")
    click.echo(f"using editor: {editor_path}")
    p = subprocess.run([editor_path, journal_path])
    return

@cli.command()
@click.pass_context
def merge(ctx):
    click.echo("This function is currently not implemented")
    # TODO: create function that resolves merge conflicts


import io
import sys
from glob import glob

from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [("pytest-args=", "a", "Arguments to pass to pytest")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ""

    def run_tests(self):
        import shlex

        # import here, cause outside the eggs aren't loaded
        import pytest

        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ) as fh:
        return fh.read()


requirements = ["jrnl", "ruamel.yaml", "click"]  # TODO: add requirements (do I want to leverage gitpython?)

# TODO: add test requirements

# upload to pypi:
#   python setup.py sdist bdist_wheel
#   python -m twine upload dist/*

setup(
    name="jrnlsync",
    version="0.1.11",    
    author="Timo Lesterhuis",
    author_email="timolesterhuis@gmail.com",
    description="Easily sync jrnl entries between different devices using git",
    long_description=read("README.rst"),
    long_description_content_type="text/x-rst",
    url="https://github.com/timolesterhuis/jrnlsync/",
    project_urls={
        'Documentation': 'https://jrnlsync.readthedocs.io/',
        #'Changelog': 'https://python-nameless.readthedocs.io/en/latest/changelog.html',
        "Source": "https://github.com/timolesterhuis/jrnlsync",
        'Issue Tracker': 'https://github.com/timolesterhuis/diagnostics/issues',
    },
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "jrnlsync = jrnlsync.sync:main",
        ]
    },
    tests_require=["pytest", "pytest-cov", "pytest-mpl"],
    cmdclass={"pytest": PyTest},
    license="MIT License",
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)